import sys
from time import sleep
import pygame
from bullet import Bullet
from alien import Alien


def check_high_score(stats, sb):
    """check to see if a new high score exists"""
    if stats.score > stats.high_score:
        stats.high_score = stats.score
        sb.prep_high_score()


def get_num_aliens(ai_settings, alien_width):
    """determine number of aliens that fit in a row"""
    available_space_x = ai_settings.screen_width - 2 * alien_width
    num_aliens_x = int(available_space_x / (2 * alien_width))
    return num_aliens_x


def get_num_rows(ai_settings, ship_height, alien_height):
    """determine the number of rows of aliens to fit the screen"""
    available_space_y = (ai_settings.screen_height - (3 * alien_height) - ship_height)
    num_rows = int(available_space_y / (2 * alien_height))
    return num_rows


def create_alien(ai_settings, screen, aliens, alien_number, row_number):
    """create alien and place in row"""
    alien = Alien(ai_settings, screen)
    alien_width = alien.rect.width
    alien.x = alien_width + 2 * alien_width * alien_number
    alien.rect.x = alien.x
    alien.rect.y = alien.rect.height + 2 * alien.rect.height * row_number
    aliens.add(alien)


def check_bullet_alien_collisions(ai_settings, screen, stats, sb, ship, aliens, bullets):
    collisions = pygame.sprite.groupcollide(bullets, aliens, True, True)
    if collisions:
        for aliens in collisions.values():
            stats.score += ai_settings.alien_points * len(aliens)
            sb.prep_score()
        check_high_score(stats, sb)

    if len(aliens) == 0:
        """Destroy existing bullets, speedup game, and create new fleet and start a new level"""
        bullets.empty()
        ai_settings.increase_speed()

        """increase level"""
        stats.level += 1
        sb.prep_level()

        create_fleet(ai_settings, screen, ship, aliens)


def update_bullets(ai_settings, screen, stats, sb, ship, aliens, bullets):
    """updates position of bullets and gets rid of old bullets"""
    bullets.update()
    """delete old bullets"""
    for bullet in bullets.copy():
        if bullet.rect.bottom <= 0:
            bullets.remove(bullet)
    """check for bullets that have hit aliens"""
    """get rid of bullets and aliens"""
    check_bullet_alien_collisions(ai_settings, screen, stats, sb, ship, aliens, bullets)


def check_fleet_edges(ai_settings, aliens):
    """check if the fleet has reached the edge"""
    for alien in aliens.sprites():
        if alien.check_edges():
            change_fleet_direction(ai_settings, aliens)
            break


def change_fleet_direction(ai_settings, aliens):
    """drops fleet and changes direction"""
    for alien in aliens.sprites():
        alien.rect.y += ai_settings.fleet_drop_speed
    ai_settings.fleet_direction *= -1


def ship_hit(ai_settings, stats, sb, screen, ship, aliens, bullets):
    """response when ship is hit by an alien"""
    if stats.ships_left > 0:
        """decrement ships left"""
        stats.ships_left -= 1

        """update scoreboard"""
        sb.prep_ships()

        """empty aliens and bullets"""
        aliens.empty()
        bullets.empty()

        """make new alien fleet and center ship"""
        create_fleet(ai_settings, screen, ship, aliens)
        ship.center_ship()

        """pause"""
        sleep(0.5)

    else:
        stats.game_active = False
        pygame.mouse.set_visible(True)


def check_aliens_bottom(ai_settings, stats, sb, screen, ship, aliens, bullets):
    """check if aliens are at the bottome of the screen and take away a life"""
    screen_rect = screen.get_rect()
    for alien in aliens.sprites():
        if alien.rect.bottom >= screen_rect.bottom:
            """same as a hit ship"""
            ship_hit(ai_settings, stats, sb, screen, ship, aliens, bullets)
            break


def update_aliens(ai_settings, screen, stats, sb, ship, aliens, bullets):
    """check if fleet has reached edge and update alien positions"""
    check_fleet_edges(ai_settings, aliens)
    aliens.update()

    """check for collisions between aliens and ship"""
    if pygame.sprite.spritecollideany(ship, aliens):
        ship_hit(ai_settings, stats, sb, screen, ship, aliens, bullets)
    """look for aliens hitting the bottom of the screen"""
    check_aliens_bottom(ai_settings, stats, sb, screen, ship, aliens, bullets)


def create_fleet(ai_settings, screen, ship, aliens):
    """create alien fleet"""
    """define amount of aliens per row"""
    """spacing between each is one alien width"""
    alien = Alien(ai_settings, screen)
    num_aliens_x = get_num_aliens(ai_settings, alien.rect.width)
    num_rows = get_num_rows(ai_settings, ship.rect.height, alien.rect.height)

    """creating rows of aliens"""
    for row_number in range(num_rows):
        for alien_number in range(num_aliens_x):
            """create alien and place it in the row"""
            create_alien(ai_settings, screen, aliens, alien_number, row_number)


def check_keydown_events(event, ai_settings, screen, ship, bullets):
    """respond to keypresses"""
    if event.key == pygame.K_RIGHT:
        """move ship right"""
        ship.moving_right = True
    elif event.key == pygame.K_LEFT:
        """move ship left"""
        ship.moving_left = True
    elif event.key == pygame.K_SPACE:
        fire_bullet(ai_settings, screen, ship, bullets)
    elif event.key == pygame.K_ESCAPE:
        sys.exit()


def fire_bullet(ai_settings, screen, ship, bullets):
    """fire bullet if not beyond the limit"""
    """create bullet and add to bullet group"""
    if len(bullets) < ai_settings.bullets_allowed:
        new_bullet = Bullet(ai_settings, screen, ship)
        bullets.add(new_bullet)


def check_keyup_events(event, ship):
    """respond to key releases"""
    if event.key == pygame.K_RIGHT:
        ship.moving_right = False
    elif event.key == pygame.K_LEFT:
        ship.moving_left = False


def check_events(ai_settings, screen, stats, sb,  play_button, ship, aliens, bullets):
    """listen for events within the game"""
    for event in pygame.event.get():
        if event.type == pygame.QUIT:
            sys.exit()
        elif event.type == pygame.KEYDOWN:
            check_keydown_events(event, ai_settings, screen, ship, bullets)
        elif event.type == pygame.KEYUP:
            check_keyup_events(event, ship)
        elif event.type == pygame.MOUSEBUTTONDOWN:
            mouse_x, mouse_y = pygame.mouse.get_pos()
            check_play_button(ai_settings, screen, stats, sb, play_button, ship, aliens, bullets, mouse_x, mouse_y)


def check_play_button(ai_settings, screen, stats, sb, play_button, ship, aliens, bullets, mouse_x, mouse_y):
    """start a new game when the user clicks play"""
    button_clicked = play_button.rect.collidepoint(mouse_x, mouse_y)
    if button_clicked and not stats.game_active:
        """reset the game settings"""
        ai_settings.initialize_dynamic_settings()
        """hide mouse cursor"""
        pygame.mouse.set_visible(False)
        """reset game stats"""
        stats.reset_stats()
        stats.game_active = True

        """reset scoreboard images"""
        sb.prep_score()
        sb.prep_high_score()
        sb.prep_level()
        sb.prep_ships()

        """empty list of aliens and bullets"""
        aliens.empty()
        bullets.empty()

        """create new fleet and center ship"""
        create_fleet(ai_settings, screen, ship, aliens)
        ship.center_ship()


def update_screen(ai_settings, screen, stats, sb, ship, aliens, bullets, play_button):
    """draw the screen each loop pass"""
    screen.fill(ai_settings.bg_color)
    """redraw bullets behind ship and aliens"""
    for bullet in bullets.sprites():
        bullet.draw_bullet()
    ship.blitme()
    aliens.draw(screen)

    """draw the score information"""
    sb.show_score()

    """Draw the play button if the game is inactive"""
    if not stats.game_active:
        play_button.draw_button()

    """make recent screen available"""
    pygame.display.flip()